import json
from datetime import timedelta
from difflib import SequenceMatcher

from panther_detection_helpers.caching import get_string_set, put_string_set

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context

okta_potentially_stolen_session_tests: list[RuleTest] = [
    RuleTest(
        name="Same device and OS",
        expected_result=False,
        mocks=[
            RuleMock(
                object_name="get_string_set",
                return_value='[\n    "263297",\n    "1.2.3.4",\n    "user_agent:Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36",\n    "CHROME",\n    "Linux"\n]\n',
            ),
        ],
        log={
            "actor": {"alternateId": "admin", "displayName": "unknown", "id": "unknown", "type": "User"},
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Dois Irmaos",
                    "country": "Brazil",
                    "geolocation": {"lat": -29.6116, "lon": -51.0933},
                    "postalCode": "93950",
                    "state": "Rio Grande do Sul",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Linux",
                    "rawUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "loginResult": "VERIFICATION_ERROR",
                    "requestId": "redacted",
                    "requestUri": "redacted",
                    "threatSuspected": "false",
                    "url": "redacted",
                    "dtHash": "kzpx58a99d2oam082rlu588wgy1mb0zfi1e1l63f9cjx4uxc455k4t6xdiwbxian",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_failed",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
            "p_any_domain_names": ["rnvtelecom.com.br"],
            "p_any_ip_addresses": ["redacted"],
            "p_event_time": "redacted",
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "redacted",
            "p_row_id": "redacted",
            "p_source_id": "redacted",
            "p_source_label": "Okta",
            "published": "redacted",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Dois Irmaos",
                            "country": "Brazil",
                            "geolocation": {"lat": -29.6116, "lon": -51.0933},
                            "postalCode": "93950",
                            "state": "Rio Grande do Sul",
                        },
                        "ip": "redacted",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 263297,
                "asOrg": "renovare telecom",
                "domain": "rnvtelecom.com.br",
                "isProxy": False,
                "isp": "renovare telecom",
            },
            "severity": "INFO",
            "transaction": {"detail": {}, "id": "redacted", "type": "WEB"},
            "uuid": "redacted",
            "version": "0",
        },
    ),
    RuleTest(
        name="Different device & ASN",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="get_string_set",
                return_value='[\n    "123456",\n    "4.3.2.1",\n    "user_agent:Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",\n    "CHROME",\n    "MacOS"\n]\n',
            ),
        ],
        log={
            "actor": {"alternateId": "admin", "displayName": "Bobert", "id": "unknown", "type": "User"},
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Dois Irmaos",
                    "country": "Brazil",
                    "geolocation": {"lat": -29.6116, "lon": -51.0933},
                    "postalCode": "93950",
                    "state": "Rio Grande do Sul",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Linux",
                    "rawUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "dtHash": "kzpx58a99d2oam082rlu588wgy1mb0zfi1e1l63f9cjx4uxc455k4t6xdiwbxian",
                    "loginResult": "VERIFICATION_ERROR",
                    "requestId": "redacted",
                    "requestUri": "redacted",
                    "threatSuspected": "false",
                    "url": "redacted",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_failed",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
            "p_any_domain_names": ["rnvtelecom.com.br"],
            "p_any_ip_addresses": ["redacted"],
            "p_event_time": "redacted",
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "redacted",
            "p_row_id": "redacted",
            "p_source_id": "redacted",
            "p_source_label": "Okta",
            "published": "redacted",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Dois Irmaos",
                            "country": "Brazil",
                            "geolocation": {"lat": -29.6116, "lon": -51.0933},
                            "postalCode": "93950",
                            "state": "Rio Grande do Sul",
                        },
                        "ip": "redacted",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 263297,
                "asOrg": "renovare telecom",
                "domain": "rnvtelecom.com.br",
                "isProxy": False,
                "isp": "renovare telecom",
            },
            "severity": "INFO",
            "transaction": {"detail": {}, "id": "redacted", "type": "WEB"},
            "uuid": "redacted",
            "version": "0",
        },
    ),
    RuleTest(
        name="Different ASN & same device",
        expected_result=False,
        mocks=[
            RuleMock(
                object_name="get_string_set",
                return_value='[\n    "654321",\n    "1.2.3.4",\n    "user_agent:Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36",\n    "CHROME",\n    "Linux"\n]\n',
            ),
        ],
        log={
            "actor": {"alternateId": "admin", "displayName": "Bobert", "id": "unknown", "type": "User"},
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Dois Irmaos",
                    "country": "Brazil",
                    "geolocation": {"lat": -29.6116, "lon": -51.0933},
                    "postalCode": "93950",
                    "state": "Rio Grande do Sul",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Linux",
                    "rawUserAgent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.135 Safari/537.36",
                },
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "loginResult": "VERIFICATION_ERROR",
                    "requestId": "redacted",
                    "requestUri": "redacted",
                    "threatSuspected": "false",
                    "url": "redacted",
                    "dtHash": "kzpx58a99d2oam082rlu588wgy1mb0zfi1e1l63f9cjx4uxc455k4t6xdiwbxian",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_failed",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
            "p_any_domain_names": ["rnvtelecom.com.br"],
            "p_any_ip_addresses": ["redacted"],
            "p_event_time": "redacted",
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "redacted",
            "p_row_id": "redacted",
            "p_source_id": "redacted",
            "p_source_label": "Okta",
            "published": "redacted",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Dois Irmaos",
                            "country": "Brazil",
                            "geolocation": {"lat": -29.6116, "lon": -51.0933},
                            "postalCode": "93950",
                            "state": "Rio Grande do Sul",
                        },
                        "ip": "redacted",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {
                "asNumber": 263297,
                "asOrg": "renovare telecom",
                "domain": "rnvtelecom.com.br",
                "isProxy": False,
                "isp": "renovare telecom",
            },
            "severity": "INFO",
            "transaction": {"detail": {}, "id": "redacted", "type": "WEB"},
            "uuid": "redacted",
            "version": "0",
        },
    ),
    RuleTest(
        name="Okta internal event should be ignored",
        expected_result=False,
        mocks=[
            RuleMock(
                object_name="get_string_set",
                return_value='[\n    "123456",\n    "4.3.2.1",\n    "user_agent:Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",\n    "CHROME",\n    "MacOS"\n]\n',
            ),
        ],
        log={
            "actor": {"alternateId": "admin", "displayName": "Bobert", "id": "unknown", "type": "User"},
            "authenticationContext": {"authenticationStep": 0, "externalSessionId": "123456789"},
            "client": {
                "device": "Unknown",
                "geographicalContext": {
                    "city": "Boardman",
                    "country": "United States",
                    "geolocation": {"lat": 45.8234, "lon": -119.7257},
                    "postalCode": "97818",
                    "state": "Oregon",
                },
                "id": "okta.b58d5b75-07d4-5f25-bf59-368a1261a405",
                "ipAddress": "44.238.82.114",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Okta-Integrations"},
                "zone": "null",
            },
            "debugContext": {
                "debugData": {
                    "loginResult": "VERIFICATION_ERROR",
                    "requestId": "redacted",
                    "requestUri": "redacted",
                    "threatSuspected": "false",
                    "url": "redacted",
                },
            },
            "displayMessage": "User login to Okta",
            "eventType": "user.session.start",
            "legacyEventType": "core.user_auth.login_failed",
            "outcome": {"reason": "VERIFICATION_ERROR", "result": "FAILURE"},
            "p_any_domain_names": ["rnvtelecom.com.br"],
            "p_any_ip_addresses": ["redacted"],
            "p_event_time": "redacted",
            "p_log_type": "Okta.SystemLog",
            "p_parse_time": "redacted",
            "p_row_id": "redacted",
            "p_source_id": "redacted",
            "p_source_label": "Okta",
            "published": "redacted",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Boardman",
                            "country": "United States",
                            "geolocation": {"lat": 45.8234, "lon": -119.7257},
                            "postalCode": "97818",
                            "state": "Oregon",
                        },
                        "ip": "44.238.82.114",
                        "version": "V4",
                    },
                ],
            },
            "securityContext": {},
            "severity": "INFO",
            "transaction": {"detail": {}, "id": "redacted", "type": "WEB"},
            "uuid": "redacted",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaPotentiallyStolenSession(Rule):
    id = "Okta.PotentiallyStolenSession-prototype"
    display_name = "Okta Potentially Stolen Session"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Okta"]
    reports = {"MITRE ATT&CK": ["TA0006:T1539"]}
    default_severity = Severity.HIGH
    default_description = (
        "This rule looks for the same session being used from two devices, indicating a compromised session token."
    )
    default_runbook = "Confirm the session is used on two devices, one of which is unknown. Lock the users Okta account and clear the users sessions in down stream apps."
    default_reference = "https://sec.okta.com/sessioncookietheft"
    summary_attributes = ["eventType", "severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = okta_potentially_stolen_session_tests
    FUZZ_RATIO_MIN = 0.95
    PREVIOUS_SESSION = {}
    # the number of days an Okta session is valid for (configured in Okta)
    SESSION_TIMEOUT = timedelta(days=1).total_seconds()
    EVENT_TYPES = ("user.authentication.sso", "user.session.start")

    def rule(self, event):
        # ensure previous session info is avaialable in the alert_context for investigation
        session_id = deep_get(event, "authenticationContext", "externalSessionId", default="unknown")
        dt_hash = deep_get(event, "debugContext", "debugData", "dtHash", default="unknown")
        # Some events by Okta admins may appear to have changed IPs
        # and user agents due to internal Okta behavior:
        # https://support.okta.com/help/s/article/okta-integrations-showing-as-rawuseragent-with-okta-ips
        # As such, we ignore certain client ids known to originate from Okta:
        # https://developer.okta.com/docs/api/openapi/okta-myaccount/myaccount/tag/OktaApplications/
        if deep_get(event, "client", "id") in ["okta.b58d5b75-07d4-5f25-bf59-368a1261a405"]:  # Admin Console
            return False
        # Filter only on app access and session start events
        if event.get("eventType") not in self.EVENT_TYPES or (session_id == "unknown" or dt_hash == "unknown"):
            return False
        key = session_id + "-" + dt_hash
        # lookup if we've previously stored the session cookie
        self.PREVIOUS_SESSION = get_string_set(key)
        # For unit test mocks we need to eval the string to a set
        if isinstance(self.PREVIOUS_SESSION, str):
            self.PREVIOUS_SESSION = set(json.loads(self.PREVIOUS_SESSION))
        # If the sessionID has not been seen before, store information about it
        if not self.PREVIOUS_SESSION:
            # clearly label the user agent string so we can find it during the comparison
            put_string_set(
                key,
                [
                    str(deep_get(event, "securityContext", "asNumber")),
                    deep_get(event, "client", "ipAddress"),
                    "user_agent:" + deep_get(event, "client", "userAgent", "rawUserAgent"),
                    deep_get(event, "client", "userAgent", "browser"),
                    deep_get(event, "client", "userAgent", "os"),
                    event.get("p_event_time"),
                    "sign_on_mode:" + deep_get(event, "debugContext", "debugData", "signOnMode", default="unknown"),
                    "threat_suspected:"
                    + deep_get(event, "debugContext", "debugData", "threat_suspected", default="unknown"),
                ],
                epoch_seconds=event.event_time_epoch() + self.SESSION_TIMEOUT,
            )
        else:
            # if the session cookie was seen before
            # we use a fuzz match to compare the current and prev user agent.
            # We cannot do a direct match since Okta can occasionally maintain
            # a session across browser upgrades.
            # the user-agent was tagged during storage so we can find it, remove that tag
            [prev_ua] = [x for x in self.PREVIOUS_SESSION if "user_agent:" in x] or ["prev_ua_not_found"]
            prev_ua = prev_ua.split("_agent:")[1]
            diff_ratio = SequenceMatcher(
                None,
                deep_get(event, "client", "userAgent", "rawUserAgent", default="ua_not_found"),
                prev_ua,
            ).ratio()
            # is this session being used from a new IP and a different browser
            if (
                str(deep_get(event, "client", "ipAddress", default="ip_not_found")) not in self.PREVIOUS_SESSION
                and diff_ratio < self.FUZZ_RATIO_MIN
            ):
                # make the fuzz ratio available in the alert context
                self.PREVIOUS_SESSION.add("Fuzz Ratio: " + str(diff_ratio))
                return True
        return False

    def title(self, event):
        return f"Potentially Stolen Okta Session - {deep_get(event, 'actor', 'displayName', default='Unknown_user')}"

    def alert_context(self, event):
        context = okta_alert_context(event)
        context["previous_session"] = str(self.PREVIOUS_SESSION)
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success

awsiam_policy_modified_tests: list[RuleTest] = [
    RuleTest(
        name="IAM Policy Change",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "DeleteGroupPolicy",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"policyName": "policy", "groupName": "group"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Not IAM Policy Change",
        expected_result=False,
        log={
            "eventVersion": "1.06",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "dynamodb.amazonaws.com",
            "eventName": "DescribeTable",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"tableName": "table"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [
                {
                    "accountId": "123456789012",
                    "type": "AWS::DynamoDB::Table",
                    "ARN": "arn:aws:dynamodb:us-west-2:123456789012:table/table",
                },
            ],
            "eventType": "AwsApiCall",
            "apiVersion": "2012-08-10",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="IAM Policy Change Error",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "NoSuchEntity",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "DeleteGroupPolicy",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"policyName": "policy", "groupName": "group"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSIAMPolicyModified(Rule):
    id = "AWS.IAM.PolicyModified-prototype"
    display_name = "IAM Policy Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "Privilege Escalation:Abuse Elevation Control Mechanism"]
    reports = {"CIS": ["3.4"], "MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "An IAM Policy was changed.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-iam-policy-modified"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/access_policies.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = awsiam_policy_modified_tests
    # API calls that are indicative of IAM Policy changes
    # Put<Entity>Policy is for inline policies.
    # these can be moved into their own rule if inline policies are of a greater concern.
    POLICY_CHANGE_EVENTS = {
        "DeleteGroupPolicy",
        "DeleteRolePolicy",
        "DeleteUserPolicy",
        "PutGroupPolicy",
        "PutRolePolicy",
        "PutUserPolicy",
        "CreatePolicy",
        "DeletePolicy",
        "CreatePolicyVersion",
        "DeletePolicyVersion",
        "AttachRolePolicy",
        "DetachRolePolicy",
        "AttachUserPolicy",
        "DetachUserPolicy",
        "AttachGroupPolicy",
        "DetachGroupPolicy",
    }

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.POLICY_CHANGE_EVENTS

    def dedup(self, event):
        return event.get("recipientAccountId")

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_workspace_data_export_created_tests: list[RuleTest] = [
    RuleTest(
        name="Workspace Admin Data Export Created",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "admin@example.io", "profileId": "11011111111111111111111"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 22:21:40.079000000",
                "uniqueQualifier": "-2833899999999999999",
            },
            "kind": "admin#reports#activity",
            "name": "CUSTOMER_TAKEOUT_CREATED",
            "parameters": {"OBFUSCATED_CUSTOMER_TAKEOUT_REQUEST_ID": "00mmmmmmmmmmmmm"},
            "type": "CUSTOMER_TAKEOUT",
        },
    ),
    RuleTest(
        name="Workspace Admin Data Export Succeeded",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "admin@example.io", "profileId": "11011111111111111111111"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-12 22:21:40.106000000",
                "uniqueQualifier": "3005999999999999999",
            },
            "kind": "admin#reports#activity",
            "name": "CUSTOMER_TAKEOUT_SUCCEEDED",
            "parameters": {"OBFUSCATED_CUSTOMER_TAKEOUT_REQUEST_ID": "00mmmmmmmmmmmmm"},
            "type": "CUSTOMER_TAKEOUT",
        },
    ),
    RuleTest(
        name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to MANAGE_ACCESS",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:06:26.303000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_CALENDAR_SETTING",
            "parameters": {
                "DOMAIN_NAME": "example.io",
                "NEW_VALUE": "MANAGE_ACCESS",
                "OLD_VALUE": "READ_WRITE_ACCESS",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
            },
            "type": "CALENDAR_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteWorkspaceDataExportCreated(Rule):
    id = "GSuite.Workspace.DataExportCreated-prototype"
    display_name = "GSuite Workspace Data Export Has Been Created"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.MEDIUM
    default_description = "A Workspace Admin Has Created a Data Export\n"
    default_reference = "https://support.google.com/a/answer/100458?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Verify the intent of this Data Export. If intent cannot be verified, then a search on the actor's other activities is advised.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_workspace_data_export_created_tests

    def rule(self, event):
        return event.get("name", "").startswith("CUSTOMER_TAKEOUT_")

    def title(self, event):
        return f"GSuite Workspace Data Export [{event.get('name', '<NO_EVENT_NAME>')}] performed by [{deep_get(event, 'actor', 'email', default='<NO_ACTOR_FOUND>')}]"



"""Global helpers for Snyk detections"""


def snyk_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = event.get("userId", "<NO_USERID>")
    a_c["action"] = event.get("event", "<NO_EVENT>")
    for pass_thru in ["orgId", "groupId"]:
        a_c[pass_thru] = event.get(pass_thru, f"<NO_{pass_thru}>".upper())
    if a_c.get("actor", "<NO_USERID>") != "<NO_USERID>" and a_c.get("groupId", "<NO_GROUPID>") != "<NO_GROUPID>":
        a_c["actor_link"] = f"https://app.snyk.io/group/{a_c.get('groupId')}/manage/member/{a_c.get('actor')}"
    return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_new_meeting_passcode_required_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Setting Turn Off",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require a passcode when scheduling new meetings: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:22:17",
        },
    ),
    RuleTest(
        name="Setting Turn On",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require a passcode when scheduling new meetings: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:22:17",
        },
    ),
    RuleTest(
        name="Automatic Sign Out Setting Disabled ",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Automatically sign users out after a specified time: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:42",
        },
    ),
]


@panther_managed
class ZoomNewMeetingPasscodeRequiredDisabled(Rule):
    default_description = "A Zoom User turned off your organization's setting to require passcodes for new meetings."
    display_name = "Zoom New Meeting Passcode Required Disabled"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0063160#:~:text=Since%20September%202022%2C%20Zoom%20requires,enforced%20for%20all%20free%20accounts"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.New.Meeting.Passcode.Required.Disabled-prototype"
    tests = zoom_new_meeting_passcode_required_disabled_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Security  - Require a passcode when scheduling new meetings: from On to Off"
        return all(
            [
                event.get("action", "<NO_ACTION>") == "Update",
                event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account",
                operation_flag == operation_detail,
            ],
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] turned off your organization's setting to require passcodes for new meetings."



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import zendesk_get_roles

zendesk_user_role_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Zendesk - Role Changed",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from Administrator to End User",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Admin Role Assigned",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskUserRoleChanged(Rule):
    id = "Zendesk.UserRoleChanged-prototype"
    display_name = "Zendesk User Role Changed"
    log_types = [LogType.ZENDESK_AUDIT]
    default_severity = Severity.INFO
    default_description = "A user's Zendesk role was changed"
    default_reference = (
        "https://support.zendesk.com/hc/en-us/articles/4408824375450-Setting-roles-and-access-in-Zendesk-Admin-Center"
    )
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_user_role_changed_tests

    def rule(self, event):
        if event.get("source_type") == "user" and event.get("action") == "update":
            # admin roles have their own handling
            if event.udm("event_type") != event_type.ADMIN_ROLE_ASSIGNED:
                _, new_role = zendesk_get_roles(event)
                return bool(new_role)
        return False

    def title(self, event):
        old_role, new_role = zendesk_get_roles(event)
        return (
            f"Actor user [{event.udm('actor_user')}] changed [{event.udm('user')}] role from {old_role} to {new_role}"
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import github_alert_context

git_hub_advanced_security_change_tests: list[RuleTest] = [
    RuleTest(
        name="Secret Scanning Disabled on a Repo",
        expected_result=True,
        log={
            "action": "repository_secret_scanning_push_protection.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Secret Scanning Disabled Org Wide",
        expected_result=True,
        log={
            "action": "secret_scanning.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Secret Scanning Disabled for New Repos",
        expected_result=True,
        log={
            "action": "secret_scanning_new_repos.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Dependabot Alerts Disabled Org Wide",
        expected_result=True,
        log={
            "action": "dependabot_alerts.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Dependabot Alerts Disabled on New Repos",
        expected_result=True,
        log={
            "action": "dependabot_alerts_new_repos.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Dependabot Disabled Org Wide",
        expected_result=True,
        log={
            "action": "dependabot_security_updates.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Dependabot Disabled on New Repos",
        expected_result=True,
        log={
            "action": "dependabot_security_updates_new_repos.disable",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Non-GitHub Adv Sec Action",
        expected_result=False,
        log={
            "action": "enterprise.config.disable_anonymous_git_access",
            "actor": "bobert",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-08-16 16:56:49.309",
            "created_at": "2022-08-16 16:56:49.309",
            "org": "an-org",
            "repo": "an-org/a-repo",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Enterprise Log - business_advanced_security.enabled",
        expected_result=False,
        log={
            "@timestamp": 1671111111111,
            "_document_id": "gAcccccccccccccccccccc",
            "action": "business_advanced_security.enabled",
            "actor": "bobert",
            "actor_ip": "12.12.12.12",
            "actor_location": {"country_code": "US"},
            "business": "example-enterprise",
            "created_at": 1671111111111,
            "operation_type": "modify",
            "user": "bobert",
        },
    ),
    RuleTest(
        name="Enterprise Log - business_advanced_security.disabled",
        expected_result=True,
        log={
            "@timestamp": 1671111111111,
            "_document_id": "gAcccccccccccccccccccc",
            "action": "business_advanced_security.disabled",
            "actor": "bobert",
            "actor_ip": "12.12.12.12",
            "actor_location": {"country_code": "US"},
            "business": "example-enterprise",
            "created_at": 1671111111111,
            "operation_type": "modify",
            "user": "bobert",
        },
    ),
]


@panther_managed
class GitHubAdvancedSecurityChange(Rule):
    id = "GitHub.Advanced.Security.Change-prototype"
    display_name = "GitHub Security Change, includes GitHub Advanced Security"
    create_alert = False
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.LOW
    default_description = "The rule alerts when GitHub Security tools (Dependabot, Secret Scanner, etc) are disabled."
    default_runbook = "Confirm with GitHub administrators and re-enable the tools as applicable."
    default_reference = "https://docs.github.com/en/code-security/getting-started/auditing-security-alerts"
    tests = git_hub_advanced_security_change_tests
    # List of actions in markdown format
    # pylint: disable=line-too-long
    # https://github.com/github/docs/blob/main/content/admin/monitoring-activity-in-your-enterprise/reviewing-audit-logs-for-your-enterprise/audit-log-events-for-your-enterprise.md
    # grep '^| `' audit-log-events-for-your-enterprise.md.txt | sed -e 's/\| //' -e 's/`//g' | awk -F\| '{if ($1 ~ /business/) {print $1}}'
    # pylint: enable=line-too-long
    # {GitHub Action: Alert Severity}
    # Bypass secret scanner push protection for a detected secret.
    # pylint: disable=line-too-long
    # The events that begin with "business" are seemingly from enterprise logs
    # business.disable_oidc  -  OIDC single sign-on was disabled for an enterprise.
    # business.disable_saml  -  SAML single sign-on was disabled for an enterprise.
    # business.disable_two_factor_requirement  -  The requirement for members to
    #    have two-factor authentication enabled to access an enterprise was disabled.
    # business.members_can_update_protected_branches.disable  -  The ability for
    #    enterprise members to update branch protection rules was disabled.
    #    Only enterprise owners can update protected branches.
    # business.referrer_override_disable  -  An enterprise owner or site administrator
    #    disabled the referrer policy override.
    # business_advanced_security.disabled  -  {% data
    #    variables.product.prodname_GH_advanced_security %}
    #    was disabled for your enterprise. For more information, see "[Managing
    #    {% data variables.product.prodname_GH_advanced_security %}
    #    features for your enterprise]
    #    (/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_advanced_security.disabled_for_new_repos  -  {% data
    #    variables.product.prodname_GH_advanced_security %} was disabled for
    #    new repositories in your enterprise. For more information, see
    #    "[Managing {% data variables.product.prodname_GH_advanced_security %} features
    #    for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_secret_scanning.disable  -  {% data variables.product.prodname_secret_scanning_caps %} was disabled for your enterprise. For more information, see "[Managing {% data variables.product.prodname_GH_advanced_security %} features for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_secret_scanning.disabled_for_new_repos  -  {% data variables.product.prodname_secret_scanning_caps %} was disabled for new repositories in your enterprise. For more information, see "[Managing {% data variables.product.prodname_GH_advanced_security %} features for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_secret_scanning_custom_pattern_push_protection.disabled  -  Push protection for a custom pattern for {% data variables.product.prodname_secret_scanning %} was disabled for your enterprise. For more information, see "[Defining custom patterns for {% data variables.product.prodname_secret_scanning %}](/code-security/secret-scanning/defining-custom-patterns-for-secret-scanning#defining-a-custom-pattern-for-an-enterprise-account)."
    # business_secret_scanning_push_protection.disable  -  Push protection for {% data variables.product.prodname_secret_scanning %} was disabled for your enterprise. For more information, see "[Managing {% data variables.product.prodname_GH_advanced_security %} features for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_secret_scanning_push_protection.disabled_for_new_repos  -  Push protection for {% data variables.product.prodname_secret_scanning %} was disabled for new repositories in your enterprise. For more information, see "[Managing {% data variables.product.prodname_GH_advanced_security %} features for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    # business_secret_scanning_push_protection_custom_message.disable  -  The custom message triggered by an attempted push to a push-protected repository was disabled for your enterprise. For more information, see "[Managing {% data variables.product.prodname_GH_advanced_security %} features for your enterprise](/admin/code-security/managing-github-advanced-security-for-your-enterprise/managing-github-advanced-security-features-for-your-enterprise)."
    #
    # There are also correlating github _org_ level events
    # org.advanced_security_policy_selected_member_disabled - An enterprise owner prevented {% data variables.product.prodname_GH_advanced_security %} features from being enabled for repositories owned by the organization. {% data reusables.advanced-security.more-information-about-enforcement-policy %}
    # pylint: enable=line-too-long
    # repository_vulnerability_alerts.disable - Dependabot alerts was disabled.
    ADV_SEC_ACTIONS = {
        "dependabot_alerts.disable": "CRITICAL",
        "dependabot_alerts_new_repos.disable": "HIGH",
        "dependabot_security_updates.disable": "CRITICAL",
        "dependabot_security_updates_new_repos.disable": "HIGH",
        "repository_secret_scanning_push_protection.disable": "HIGH",
        "secret_scanning.disable": "CRITICAL",
        "secret_scanning_new_repos.disable": "HIGH",
        "bypass": "MEDIUM",
        "business.disable_oidc": "CRITICAL",
        "business.disable_saml": "CRITICAL",
        "business.disable_two_factor_requirement": "CRITICAL",
        "business.members_can_update_protected_branches.disable": "MEDIUM",
        "business.referrer_override_disable": "MEDIUM",
        "business_advanced_security.disabled": "CRITICAL",
        "business_advanced_security.disabled_for_new_repos": "HIGH",
        "business_secret_scanning.disable": "CRITICAL",
        "business_secret_scanning.disabled_for_new_repos": "CRITICAL",
        "business_secret_scanning_custom_pattern_push_protection.disabled": "HIGH",
        "business_secret_scanning_push_protection.disable": "CRITICAL",
        "business_secret_scanning_push_protection.disabled_for_new_repos": "HIGH",
        "business_secret_scanning_push_protection_custom_message.disable": "HIGH",
        "org.advanced_security_disabled_for_new_repos": "HIGH",
        "org.advanced_security_disabled_on_all_repos": "CRITICAL",
        "org.advanced_security_policy_selected_member_disabled": "HIGH",
        "repo.advanced_security_disabled": "CRITICAL",
        "repo.advanced_security_policy_selected_member_disabled": "HIGH",
        "repository_vulnerability_alerts.disable": "HIGH",
    }
    # Use the per action severity configured above

    def rule(self, event):
        return event.get("action", "") in self.ADV_SEC_ACTIONS

    def title(self, event):
        action = event.get("action", "")
        advanced_sec_text = ""
        # https://docs.github.com/en/get-started/learning-about-github/about-github-advanced-security#about-advanced-security-features
        if "advanced_security" in action or "secret_scanning" in action:
            advanced_sec_text = "Advanced "
        return f"Change detected to GitHub {advanced_sec_text}Security - {event.get('action', '')}"

    def alert_context(self, event):
        return github_alert_context(event)

    def severity(self, event):
        return self.ADV_SEC_ACTIONS.get(event.get("action", ""), "Low")

    def dedup(self, event):
        # 1. Actor
        # 2. Action
        # We should dedup on actor - action
        actor = event.get("actor", "<NO_ACTOR>")
        action = event.get("action", "<NO_ACTION>")
        return "_".join([actor, action])



from panther_core.immutable import ImmutableList

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

git_lab_production_password_reset_multiple_emails_tests: list[RuleTest] = [
    RuleTest(
        name="not a password reset",
        expected_result=False,
        log={
            "params": [
                {"key": "authenticity_token", "value": "[FILTERED]"},
                {"key": "user", "value": {"email": ["peter@example.com", "bob@example.com"]}},
            ],
            "path": "/cats",
        },
    ),
    RuleTest(
        name="one email",
        expected_result=False,
        log={
            "params": [
                {"key": "authenticity_token", "value": "[FILTERED]"},
                {"key": "user", "value": {"email": ["bob@example.com"]}},
            ],
            "path": "/users/password",
        },
    ),
    RuleTest(
        name="multiple emails",
        expected_result=True,
        log={
            "params": [
                {"key": "authenticity_token", "value": "[FILTERED]"},
                {"key": "user", "value": {"email": ["peter@example.com", "bob@example.com"]}},
            ],
            "path": "/users/password",
        },
    ),
]


@panther_managed
class GitLabProductionPasswordResetMultipleEmails(Rule):
    id = "GitLab.Production.Password.Reset.Multiple.Emails-prototype"
    display_name = "CVE-2023-7028 - GitLab Production Password Reset Multiple Emails"
    log_types = [LogType.GITLAB_PRODUCTION]
    tags = ["GitLab", "CVE-2023-7028"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195", "TA0001:T1190", "TA0003:T1098"]}
    default_severity = Severity.HIGH
    default_description = "Attackers are exploiting a Critical (CVSS 10.0) GitLab vulnerability in which user account password reset emails could be delivered to an unverified email address."
    default_reference = "https://about.gitlab.com/releases/2024/01/11/critical-security-release-gitlab-16-7-2-released/"
    tests = git_lab_production_password_reset_multiple_emails_tests

    def rule(self, event):
        path = event.get("path", default="")
        if path != "/users/password":
            return False
        params = event.get("params", default=[])
        for param in params:
            if param.get("key") == "user":
                email = deep_get(param, "value", "email", default=[])
                if isinstance(email, ImmutableList) and len(email) > 1:
                    return True
        return False

    def title(self, event):
        emails = event.deep_get("detail", "target_details", default="")
        return f"Someone tried to reset your password with multiple emails :{emails}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_high_risk_failed_login_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "event_type_id": "6",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="Failed High Risk Login",
        expected_result=True,
        log={
            "event_type_id": "6",
            "risk_score": 55,
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginHighRiskFailedLogin(Rule):
    id = "OneLogin.HighRiskFailedLogin-prototype"
    display_name = "OneLogin Failed High Risk Login"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin"]
    default_severity = Severity.LOW
    default_description = "A OneLogin attempt with a high risk factor (>50) resulted in a failed authentication."
    default_reference = "https://resources.onelogin.com/OneLogin_RiskBasedAuthentication-WP-v5.pdf"
    default_runbook = "Investigate why this user login is tagged as high risk as well as whether this was caused by expected user activity."
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_high_risk_failed_login_tests

    def rule(self, event):
        # check risk associated with this event
        if event.get("risk_score", 0) > 50:
            # a failed authentication attempt with high risk
            return str(event.get("event_type_id")) == "6"
        return False

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] failed a high risk login attempt"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

app_omni_alert_passthrough_tests: list[RuleTest] = [
    RuleTest(
        name="Alert Type Severity 2",
        expected_result=True,
        log={
            "appomni": {
                "alert": {"channel": "prod"},
                "event": {
                    "dataset": "appomni_alert",
                    "id": "2ae1e281-4df1-5d26-81e2-7b75589e5dd4",
                    "sortable_event_id": "01HQ6JKJ5VE68CAT71JM27Z1D2",
                    "sortable_ingest_id": "01HQ6KGT23SN874A9ATHZCM1JH",
                },
                "organization": {"id": 285},
            },
            "event": {"created": "2024-02-21T19:50:42.499Z", "kind": "alert", "severity": 2},
            "message": "Security issue detected in GitHub repository 'appomni/ao_factory_interfaces'",
            "related": {
                "event": ["cf8e782f-1657-5a4e-bdc2-cff1d147c912"],
                "services": {"id": [12477], "type": ["github"]},
            },
            "rule": {
                "name": "Repository Security Issue Detected",
                "ruleset": "1423ff39-3250-4d53-aafb-142e740668bd",
                "threat": {
                    "framework": "MITRE ATT&CK",
                    "tactic": {"id": ["TA0001"], "name": ["Initial Access"]},
                    "technique": {"id": ["T1195"], "name": ["Supply Chain Compromise"]},
                },
                "uuid": "6d873f19-4847-4412-9b70-6dca598ee64c",
                "version": "1",
            },
            "timestamp": "2024-02-21T19:34:44.155Z",
            "version": "2.0.0",
        },
    ),
    RuleTest(
        name="Event Type",
        expected_result=False,
        log={
            "appomni": {
                "event": {
                    "collected_time": "2024-02-28T19:53:34.266Z",
                    "dataset": "ao_auditlogs",
                    "id": "e4431a54-e57d-5cab-8b24-af194d49ebec",
                    "ingestion_time": "2024-02-28T19:53:34.298Z",
                },
                "organization": {"id": 6},
                "service": {"account_id": "6", "id": 0, "name": "AppOmni", "type": "appomni"},
            },
            "event": {
                "action": "update_token",
                "category": ["authentication"],
                "code": "access_token_refreshed_refreshtoken",
                "created": "2024-02-28T19:53:34.266Z",
                "dataset": "ao_auditlogs",
                "id": "b90b4447-ae6a-4257-95fe-a3f9c5577158",
                "ingested": "2024-02-28T19:53:34.298Z",
                "kind": "event",
                "module": "appomni",
                "original": '{"action_at":"2024-02-28T19:53:34.256900+00:00","action_data":{"md_kind":"core.aoaudit.auditdata","md_version":1},"action_type":"access_token_refreshed_refreshtoken","log_id":"b90b4447-ae6a-4257-95fe-a3f9c5577158","org_id":6,"perspective_id":1487,"service_id":34,"service_type":"workday"}',
                "type": ["change"],
            },
            "timestamp": "2024-02-28T19:53:34.256Z",
            "version": "2.0.0",
        },
    ),
    RuleTest(
        name="External App Install - Severity 3",
        expected_result=True,
        log={
            "@timestamp": "2024-02-26T18:02:09.044Z",
            "appomni": {
                "alert": {"channel": "prod"},
                "event": {
                    "dataset": "appomni_alert",
                    "id": "e927e832-bfb1-55d7-9159-0e5cd84dcc65",
                    "sortable_event_id": "01HQK99M8MZKWGZG24B5WV4JDK",
                    "sortable_ingest_id": "01HQK9DFC5DS5MYM0YEFFW7PF8",
                },
                "organization": {"id": 6},
            },
            "event": {"created": "2024-02-26T18:04:15.109Z", "kind": "alert", "severity": 3},
            "message": "An external application has been installed by appomni_int_justinz in Workday",
            "related": {
                "event": ["cb786453-a105-5438-97a6-903d15e0cb7e"],
                "ip": ["71.218.228.62"],
                "services": {"id": [34], "type": ["workday"]},
                "user": ["appomni_int_justinz"],
            },
            "rule": {
                "name": "External Application Installed",
                "ruleset": "1423ff39-3250-4d53-aafb-142e740668bd",
                "threat": {
                    "framework": "MITRE ATT&CK",
                    "tactic": {
                        "id": ["TA0005", "TA0008", "TA0010"],
                        "name": ["Defense Evasion", "Lateral Movement", "Exfiltration"],
                    },
                    "technique": {
                        "id": ["T1550", "T1550", "T1567"],
                        "name": [
                            "Use Alternate Authentication Material",
                            "Use Alternate Authentication Material",
                            "Exfiltration Over Web Service",
                        ],
                    },
                },
                "uuid": "2aadaafd-4ec5-4a09-be6e-c2d70b555d19",
                "version": "1",
            },
            "version": "2.0.0",
        },
    ),
    RuleTest(
        name="Multiple Services",
        expected_result=True,
        log={
            "appomni": {
                "alert": {"channel": "prod"},
                "event": {
                    "dataset": "appomni_alert",
                    "id": "2ae1e281-4df1-5d26-81e2-7b75589e5dd4",
                    "sortable_event_id": "01HQ6JKJ5VE68CAT71JM27Z1D2",
                    "sortable_ingest_id": "01HQ6KGT23SN874A9ATHZCM1JH",
                },
                "organization": {"id": 285},
            },
            "event": {"created": "2024-02-21T19:50:42.499Z", "kind": "alert", "severity": 2},
            "message": "Security issue detected in GitHub repository 'appomni/ao_factory_interfaces'",
            "related": {
                "event": ["cf8e782f-1657-5a4e-bdc2-cff1d147c912"],
                "services": {"id": [12477], "type": ["github", "workday"]},
            },
            "rule": {
                "name": "Repository Security Issue Detected",
                "ruleset": "1423ff39-3250-4d53-aafb-142e740668bd",
                "threat": {
                    "framework": "MITRE ATT&CK",
                    "tactic": {"id": ["TA0001"], "name": ["Initial Access"]},
                    "technique": {"id": ["T1195"], "name": ["Supply Chain Compromise"]},
                },
                "uuid": "6d873f19-4847-4412-9b70-6dca598ee64c",
                "version": "1",
            },
            "timestamp": "2024-02-21T19:34:44.155Z",
            "version": "2.0.0",
        },
    ),
]


@panther_managed
class AppOmniAlertPassthrough(Rule):
    id = "AppOmni.Alert.Passthrough-prototype"
    display_name = "AppOmni Alert Passthrough"
    log_types = [LogType.APPOMNI_ALERTS]
    default_severity = Severity.MEDIUM
    reports = {
        "MITRE ATT&CK": [
            "TA0001:T1566",
            "TA0001:T1528",
            "TA0001:T1190",
            "TA0001:T1078",
            "TA0001:T1199",
            "TA0004:T1548",
            "TA0005:T1562",
            "TA0005:T1090",
            "TA0005:T1564",
            "TA0005:T1556",
            "TA0005:T1550",
            "TA0005:T1078",
            "TA0006:T1110",
            "TA0006:T1111",
            "TA0006:T1550",
            "TA0006:T1528",
            "TA0006:T1552",
            "TA0006:T1539",
            "TA0040:T1486",
            "TA0040:T1565",
            "TA0040:T1485",
            "TA0040:T1531",
            "TA0002:T1204",
            "TA0003:T1114",
            "TA0003:T1098",
            "TA0003:T1556",
            "TA0003:T1078",
            "TA0003:T1136",
            "TA0004:T1484",
            "TA0007:T1518",
            "TA0007:T1087",
            "TA0008:T1550",
            "TA0042:T1608",
            "TA0009:T1530",
            "TA0009:T1213",
            "TA0009:T1114",
            "TA0004:T1078",
            "TA0010:T1537",
            "TA0010:T1567",
        ],
    }
    tests = app_omni_alert_passthrough_tests
    SEV_DICT = {0: "Critical", 1: "High", 2: "Medium", 3: "Low", 4: "Info"}

    def rule(self, event):
        # Only alert where event.kind == "alert"
        if event.deep_get("event", "kind") == "alert":
            return True
        return False

    def title(self, event):
        # Create title that includes severity and message
        sev = self.SEV_DICT.get(event.deep_get("event", "severity"))
        # Use type service in title if only one field, label as 'Multiple Services' if more than one.
        if len(event.deep_get("related", "services", "type", default=[])) == 1:
            service = event.deep_get("related", "services", "type")[0]
        else:
            service = "Multiple Services"
        return f"[{sev}] - {service} - {event.get('message')}"

    def severity(self, event):
        # Update Panther alert severity based on severity from AppOmni Alert
        return self.SEV_DICT[event.deep_get("event", "severity", default=4)]

    def dedup(self, event):
        # Dedup by the events alert id, make sure we alert each time a new AppOmni alert is logged
        return f"Event ID: {event.deep_get('appomni', 'event', 'id')}"

    def alert_context(self, event):
        # 'Threat' and 'related' data to be included in the alert sent to the alert destination
        return {"threat": event.deep_get("rule", "threat"), "related": event.deep_get("related")}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import duo_alert_context

duo_admin_policy_updated_tests: list[RuleTest] = [
    RuleTest(
        name="Policy Update",
        expected_result=True,
        log={
            "action": "policy_update",
            "description": '{"adaptive_auth_display_unit": "days", "trusted_mobile_endpoint_policy": "no action", "adaptive_auth_hours": 0, "admin_email": "homer.simpson@simpsons.com", "allow_factor_u2f": false, "device_certificate_policy": "no action", "allow_factor_phone": false, "local_trusted_sessions_display_val": 0, "allow_adaptive_auth": false, "local_trusted_sessions_display_unit": "days", "allow_factor_sms": false}',
            "isotimestamp": "2022-02-21 21:48:48",
            "object": "Global Policy",
            "timestamp": "2022-02-21 21:48:48",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Other event",
        expected_result=False,
        log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-123", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-07-02 18:31:25",
            "timestamp": "2021-07-02 18:31:25",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminPolicyUpdated(Rule):
    default_description = "A Duo Administrator updated a Policy, which governs how users authenticate."
    display_name = "Duo Admin Policy Updated"
    default_reference = "https://duo.com/docs/policy#authenticators-policy-settings"
    default_severity = Severity.MEDIUM
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.Policy.Updated-prototype"
    tests = duo_admin_policy_updated_tests

    def rule(self, event):
        return event.get("action") == "policy_update"

    def title(self, event):
        return f"Duo: [{event.get('username', '<username_not_found>')}] updated [{event.get('object', 'Duo Policy')}]."

    def alert_context(self, event):
        return duo_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zendesk_user_assumption_tests: list[RuleTest] = [
    RuleTest(
        name="User assumption settings changed",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Account Assumption",
            "action": "update",
            "change_description": "Changed",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Credit Card Redaction On",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Credit Card Redaction",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskUserAssumption(Rule):
    id = "Zendesk.UserAssumption-prototype"
    display_name = "Enabled Zendesk Support to Assume Users"
    log_types = [LogType.ZENDESK_AUDIT]
    tags = ["Zendesk", "Lateral Movement:Use Alternate Authentication Material"]
    reports = {"MITRE ATT&CK": ["TA0008:T1550"]}
    default_severity = Severity.MEDIUM
    default_description = "User enabled or disabled zendesk support user assumption."
    default_runbook = (
        "Investigate whether allowing zendesk support to assume users is necessary. If not, disable the feature.\n"
    )
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408894200474-Assuming-end-users#:~:text=In%20Support%2C%20click%20the%20Customers,user%20in%20the%20information%20dialog"
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_user_assumption_tests
    USER_SUSPENSION_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "account_setting"
            and event.get("action", "") in self.USER_SUSPENSION_ACTIONS
            and (event.get("source_label", "").lower() in {"account assumption", "assumption duration"})
        )

    def title(self, event):
        return f"A user [{event.udm('actor_user')}] updated zendesk support user assumption settings"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tines_helpers import tines_alert_context

tines_story_items_destruction_tests: list[RuleTest] = [
    RuleTest(
        name="Detection Trigger",
        expected_result=True,
        log={
            "created_at": "2023-06-13 15:14:46",
            "id": 1234,
            "operation_name": "StoryItemsDestruction",
            "p_source_label": "tines-log-source-name",
            "request_ip": "98.224.225.84",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (  KHTML, like Gecko) Chrome/114.0.0.0 Safari/537.36",
            "tenant_id": "1337",
            "user_email": "user@email.com",
            "user_id": "7331",
            "user_name": "Tines User Person",
        },
    ),
    RuleTest(
        name="Tines Login",
        expected_result=False,
        log={
            "created_at": "2023-05-17 14:45:19",
            "id": 7888888,
            "operation_name": "Login",
            "p_source_label": "tines-log-source-name",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
]


@panther_managed
class TinesStoryItemsDestruction(Rule):
    id = "Tines.Story.Items.Destruction-prototype"
    display_name = "Tines Story Items Destruction"
    log_types = [LogType.TINES_AUDIT]
    tags = ["Tines"]
    default_severity = Severity.INFO
    default_description = "A user has destroyed a story item"
    default_runbook = (
        "Possible data destruction. Please reach out to the user and confirm this was done for valid business reasons."
    )
    default_reference = "https://www.tines.com/docs/stories"
    tests = tines_story_items_destruction_tests

    def rule(self, event):
        return deep_get(event, "operation_name", default="<NO_OPERATION_NAME>") == "StoryItemsDestruction"

    def title(self, event):
        operation = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        user = deep_get(event, "user_email", default="<NO_USER_EMAIL>")
        tines_instance = deep_get(event, "p_source_label", default="<NO_SOURCE_LABEL>")
        return f"Tines [{operation}] performed by [{user}] on [{tines_instance}]."

    def alert_context(self, event):
        return tines_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_phishable_mfa_method_tests: list[RuleTest] = [
    RuleTest(
        name="Dropbox Phishable MFA",
        expected_result=True,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "DROPBOX",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "EMAIL_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": None,
        },
    ),
    RuleTest(
        name="Google Workspace Phishable MFA",
        expected_result=True,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "GOOGLE_WORKSPACE",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "EMAIL_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": None,
        },
    ),
    RuleTest(
        name="No MFA Enabled",
        expected_result=False,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": [],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": [],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "timestamp": 1707775049.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityPhishableMFAMethod(Rule):
    id = "Push.Security.Phishable.MFA.Method-prototype"
    display_name = "Push Security Phishable MFA Method"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO
    tests = push_security_phishable_mfa_method_tests
    identity_providers = ("MICROSOFT_365", "GOOGLE_WORKSPACE", "OKTA", "JUMPCLOUD", "PING")
    phishable_mfa = ("EMAIL_OTP", "PHONE_CALL", "SMS", "APP_PASSWORD")

    def rule(self, event):
        if event.get("object") != "ACCOUNT":
            return False
        mfa_methods = event.deep_get("new", "mfaMethods")
        for method in mfa_methods:
            if method in self.phishable_mfa:
                return True
        return False

    def severity(self, event):
        if event.deep_get("new", "appType") in self.identity_providers:
            return "HIGH"
        return "INFO"

    def title(self, event):
        mfa_methods = ", ".join(event.deep_get("new", "mfaMethods", default="No MFA"))
        new_email = event.deep_get("new", "email")
        app_type = event.deep_get("new", "appType", default=[])
        return f"{new_email} using phisbable MFA method with {app_type}.             MFA methods enabled: {mfa_methods}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpiam_org_folder_iam_changes_tests: list[RuleTest] = [
    RuleTest(
        name="Terraform User Agent",
        expected_result=True,
        log={
            "insertId": "-lmjke7dbt7y",
            "logName": "organizations/888888888888/logs/cloudaudit.googleapis.com%2Factivity",
            "p_log_type": "GCP.AuditLog",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "terraform@platform.iam.gserviceaccount.com",
                    "principalSubject": "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/platform/serviceAccounts/terraform@platform.iam.gserviceaccount.com/keys/bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "resourcemanager.organizations.setIamPolicy",
                        "resource": "organizations/888888888888",
                        "resourceAttributes": {
                            "name": "organizations/888888888888",
                            "service": "cloudresourcemanager.googleapis.com",
                            "type": "cloudresourcemanager.googleapis.com/Organization",
                        },
                    },
                ],
                "methodName": "SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {
                        "bindings": [
                            {
                                "members": [
                                    "joey.jojo@example.com",
                                    "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                                ],
                                "role": "roles/owner",
                            },
                        ],
                        "etag": "BwXcRFUAtX4=",
                    },
                    "resource": "organizations/888888888888",
                    "updateMask": "bindings,etag,auditConfigs",
                },
                "requestMetadata": {
                    "callerIP": "100.100.100.100",
                    "callerSuppliedUserAgent": "Terraform/0.13.2 terraform-provider-google/3.90.1",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resourceName": "organizations/888888888888",
                "response": {
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                    "bindings": [
                        {
                            "members": [
                                "joey.jojo@example.com",
                                "serviceAccount:terraform@platform.iam.gserviceaccount.com",
                            ],
                            "role": "roles/owner",
                        },
                    ],
                    "etag": "BwXeRCtKxCw=",
                },
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {"action": "ADD", "member": "user:backdoor@example.com", "role": "roles/owner"},
                        ],
                    },
                },
                "serviceName": "cloudresourcemanager.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2022-05-05 14:00:49.450798551",
            "resource": {"labels": {"organization_id": "888888888888"}, "type": "organization"},
            "severity": "NOTICE",
            "timestamp": "2022-05-05 14:00:48.814294000",
        },
    ),
    RuleTest(
        name="Manual Change",
        expected_result=True,
        log={
            "insertId": "-yoga2udnx8s",
            "logName": "organizations/888888888888/logs/cloudaudit.googleapis.com%2Factivity",
            "p_log_type": "GCP.AuditLog",
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "chris@example.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "resourcemanager.organizations.setIamPolicy",
                        "resource": "organizations/888888888888",
                        "resourceAttributes": {
                            "name": "organizations/888888888888",
                            "service": "cloudresourcemanager.googleapis.com",
                            "type": "cloudresourcemanager.googleapis.com/Organization",
                        },
                    },
                ],
                "methodName": "SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "etag": "BwXYPRNtbqo=",
                    "policy": {
                        "bindings": [
                            {
                                "members": [
                                    "user:chris@example.com",
                                    "serviceAccount:diana@platform.iam.gserviceaccount.com",
                                ],
                                "role": "roles/owner",
                            },
                        ],
                    },
                },
                "receiveTimestamp": "2022-02-17T22:52:03.190032712Z",
                "requestMetadata": {
                    "callerIp": "38.38.38.38",
                    "callerSuppliedUserAgent": "Mozilla/5.0 Chrome/98.0.4758.102",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resource": {"labels": {"organization_id": "888888888888"}, "type": "organization"},
                "resourceName": "organizations/888888888888",
                "response": {
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                    "bindings": [{"members": ["user:chris@example.com"], "role": "roles/owner"}],
                    "etag": "BwXYPp1Xs/Y=",
                },
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {
                                "action": "REMOVE",
                                "member": "serviceAccount:diana@platform.iam.gserviceaccount.com",
                                "role": "roles/owner",
                            },
                        ],
                    },
                },
            },
            "serviceName": "cloudresourcemanager.googleapis.com",
            "severity": "NOTICE",
            "status": {},
            "timestamp": "2022-02-17T22:52:02.692083Z",
        },
    ),
]


@panther_managed
class GCPIAMOrgFolderIAMChanges(Rule):
    id = "GCP.IAM.OrgFolderIAMChanges-prototype"
    display_name = "GCP Org or Folder Policy Was Changed Manually"
    dedup_period_minutes = 1440
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Identity & Access Management"]
    reports = {"GCP_CIS_1.3": ["1.8"], "MITRE ATT&CK": ["Privilege Escalation:Valid Accounts"]}
    default_severity = Severity.HIGH
    default_description = "Alert if a GCP Org or Folder Policy Was Changed Manually.\n"
    default_runbook = "Contact the party that made the change. If it was intended to be temporary, ask for a window for rollback (< 24 hours). If it must be permanent, ask for change-management doc explaining why it was needed. Direct them to make the change in Terraform to avoid automated rollback. Grep for google_org and google_folder in terraform repos for places to put your new policy bindings.\n"
    default_reference = "https://cloud.google.com/iam/docs/granting-changing-revoking-access"
    summary_attributes = ["severity", "p_any_ip_addresses"]
    tests = gcpiam_org_folder_iam_changes_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        logname = deep_get(event, "logName")
        return (
            deep_get(event, "protoPayload", "methodName") == "SetIamPolicy"
            and (logname.startswith("organizations") or logname.startswith("folder"))
            and logname.endswith("/logs/cloudaudit.googleapis.com%2Factivity")
        )

    def title(self, event):
        # use unified data model field in title
        return f"{event.get('p_log_type')}: [{event.udm('actor_user')}] made manual changes to Org policy"

    def alert_context(self, event):
        return {
            "actor": event.udm("actor_user"),
            "policy_change": deep_get(event, "protoPayload", "serviceData", "policyDelta"),
            "caller_ip": deep_get(event, "protoPayload", "requestMetadata", "callerIP"),
            "user_agent": deep_get(event, "protoPayload", "requestMetadata", "callerSuppliedUserAgent"),
        }

    def severity(self, event):
        if (
            deep_get(event, "protoPayload", "requestMetadata", "callerSuppliedUserAgent").lower().find("terraform")
            != -1
        ):
            return "INFO"
        return "HIGH"



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcp_logging_sink_modified_tests: list[RuleTest] = [
    RuleTest(
        name="logging-sink.modifed-should-alert",
        expected_result=True,
        log={
            "insertid": "6ns26jclap",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.update",
                        "resource": "projects/test-project-123456/sinks/test-1",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/sinks/test-1",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.UpdateSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.UpdateSinkRequest",
                    "sink": {
                        "description": "test",
                        "destination": "logging.googleapis.com/projects/test-project-123456/locations/global/buckets/testloggingbucket",
                        "exclusions": [{"filter": "*", "name": "excludeall"}],
                        "name": "test-1",
                    },
                    "sinkName": "projects/test-project-123456/sinks/test-1",
                    "uniqueWriterIdentity": True,
                    "updateMask": "exclusions",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:39:07.289670886Z"},
                },
                "resourceName": "projects/test-project-123456/sinks/test-1",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2023-05-23 19:39:07.924",
            "resource": {
                "labels": {"destination": "", "name": "test-1", "project_id": "test-project-123456"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:39:07.272",
        },
    ),
    RuleTest(
        name="logging-sink.non-modified-should-not-alert",
        expected_result=False,
        log={
            "insertid": "6ns26jclap",
            "logname": "projects/test-project-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@domain.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.list",
                        "resource": "projects/test-project-123456/sinks/test-1",
                        "resourceAttributes": {
                            "name": "projects/test-project-123456/sinks/test-1",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.ListSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.ListSinkRequest",
                    "sink": {
                        "description": "test",
                        "destination": "logging.googleapis.com/projects/test-project-123456/locations/global/buckets/testloggingbucket",
                        "exclusions": [{"filter": "*", "name": "excludeall"}],
                        "name": "test-1",
                    },
                    "sinkName": "projects/test-project-123456/sinks/test-1",
                    "uniqueWriterIdentity": True,
                    "updateMask": "exclusions",
                },
                "requestMetadata": {
                    "callerIP": "12.12.12.12",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-05-23T19:39:07.289670886Z"},
                },
                "resourceName": "projects/test-project-123456/sinks/test-1",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2023-05-23 19:39:07.924",
            "resource": {
                "labels": {"destination": "", "name": "test-1", "project_id": "test-project-123456"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-05-23 19:39:07.272",
        },
    ),
]


@panther_managed
class GCPLoggingSinkModified(Rule):
    display_name = "GCP Logging Sink Modified"
    id = "GCP.Logging.Sink.Modified-prototype"
    default_severity = Severity.INFO
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Logging", "Sink", "Infrastructure"]
    default_description = "This rule detects modifications to GCP Log Sinks.\n"
    default_runbook = "Ensure that the modification was valid or expected. Adversaries may do this to exfiltrate logs or evade detection.\n"
    default_reference = "https://cloud.google.com/logging/docs"
    tests = gcp_logging_sink_modified_tests

    def rule(self, event):
        method_pattern = "(?:\\w+\\.)*v\\d\\.(?:ConfigServiceV\\d\\.(?:UpdateSink))"
        match = re.search(method_pattern, deep_get(event, "protoPayload", "methodName", default=""))
        return match is not None

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        resource = deep_get(event, "protoPayload", "resourceName", default="<RESOURCE_NOT_FOUND>")
        return f"[GCP]: [{actor}] updated logging sink [{resource}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_ami_modified_for_public_access_tests: list[RuleTest] = [
    RuleTest(
        name="AMI Made Public",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifyImageAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "launchPermission",
                "imageId": "ami-1111",
                "launchPermission": {"add": {"items": [{"group": "all"}]}},
            },
            "responseElements": {"_return": True},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="AMI Not Made Public",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifyImageAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "launchPermission",
                "imageId": "ami-1111",
                "launchPermission": {"add": {"items": [{"group": "not-all"}]}},
            },
            "responseElements": {"_return": True},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="AMI Launch Permissions Not Modified",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifyImageAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "someThing",
                "imageId": "ami-1111",
                "someThing": {"add": {"items": [{"group": "all"}]}},
            },
            "responseElements": {"_return": True},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="AMI Added to User",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ModifyImageAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "launchPermission",
                "imageId": "ami-1111",
                "launchPermission": {"add": {"items": [{"userId": "bob"}]}},
            },
            "responseElements": {"_return": True},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Error Making AMI Public",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "UnauthorizedOperation",
            "eventID": "1111",
            "eventName": "ModifyImageAttribute",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {
                "attributeType": "launchPermission",
                "imageId": "ami-1111",
                "launchPermission": {"add": {"items": [{"group": "all"}]}},
            },
            "responseElements": {"_return": True},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailAMIModifiedForPublicAccess(Rule):
    id = "AWS.CloudTrail.AMIModifiedForPublicAccess-prototype"
    display_name = "Amazon Machine Image (AMI) Modified to Allow Public Access"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Exfiltration:Transfer Data to Cloud Account"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0010:T1537"]}
    default_description = "An Amazon Machine Image (AMI) was modified to allow it to be launched by anyone. Any sensitive configuration or application data stored in the AMI's block devices is at risk.\n"
    default_runbook = "Determine if the AMI is intended to be publicly accessible. If not, first modify the AMI to not be publicly accessible then change any sensitive data stored in the block devices associated to the AMI (as they may be compromised).\n"
    default_reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_ami_modified_for_public_access_tests

    def rule(self, event):
        # Only check successful ModiyImageAttribute events
        if not aws_cloudtrail_success(event) or event.get("eventName") != "ModifyImageAttribute":
            return False
        added_perms = deep_get(event, "requestParameters", "launchPermission", "add", "items", default=[])
        for item in added_perms:
            if item.get("group") == "all":
                return True
        return False

    def alert_context(self, event):
        return aws_rule_context(event)



from ipaddress import ip_address

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_cloud_trail_unauthorized_api_call_tests: list[RuleTest] = [
    RuleTest(
        name="Unauthorized API Call from Within AWS (IP)",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "3.10.107.144",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:Action on resource: arn:aws:iam::123456789012:resource",
            "userAgent": "sqs.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Unauthorized API Call from Within AWS (FQDN)",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "sqs.amazonaws.com",
            "errorCode": "AccessDenied",
            "errorMessage": "User: arn:aws:iam::123456789012:user/tester is not authorized to perform: iam:Action on resource: arn:aws:iam::123456789012:resource",
            "userAgent": "sqs.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Authorized API Call",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "userName": "tester",
                "sessionContext": {"attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"}},
                "invokedBy": "signin.amazonaws.com",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateServiceLinkedRole",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "signin.amazonaws.com",
            "requestParameters": None,
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSCloudTrailUnauthorizedAPICall(Rule):
    id = "AWS.CloudTrail.UnauthorizedAPICall-prototype"
    display_name = "Monitor Unauthorized API Calls"
    dedup_period_minutes = 1440
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Discovery:Cloud Service Discovery"]
    reports = {"CIS": ["3.1"], "MITRE ATT&CK": ["TA0007:T1526"]}
    default_severity = Severity.INFO
    default_description = "An unauthorized AWS API call was made"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-unauthorized-api-call"
    default_reference = "https://amzn.to/3aOukaA"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    threshold = 20
    tests = aws_cloud_trail_unauthorized_api_call_tests
    # Do not alert on these access denied errors for these events.
    # Events could be exceptions because they are particularly noisy and provide little to no value,
    # or because they are expected as part of the normal operating procedure for certain tools.
    # Noisy, doesn't really provide any actionable info
    # The audit role hits this when scanning locked down resources
    EVENT_EXCEPTIONS = {"DescribeEventAggregates", "ListResourceTags"}

    def rule(self, event):
        # Validate the request came from outside of AWS
        try:
            ip_address(event.get("sourceIPAddress"))
        except ValueError:
            return False
        return event.get("errorCode") == "AccessDenied" and event.get("eventName") not in self.EVENT_EXCEPTIONS

    def dedup(self, event):
        return deep_get(event, "userIdentity", "principalId", default="<UNKNOWN_PRINCIPAL>")

    def title(self, event):
        return f"Access denied to {deep_get(event, 'userIdentity', 'type')} [{self.dedup(event)}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_user_access_key_created_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - User Access Key Created",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "public_key.create",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - User Access Key Deleted",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "public_key.delete",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GitHubUserAccessKeyCreated(Rule):
    id = "GitHub.User.AccessKeyCreated-prototype"
    display_name = "GitHub User Access Key Created"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Persistence:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0003:T1078"]}
    default_reference = "https://docs.github.com/en/authentication/connecting-to-github-with-ssh/generating-a-new-ssh-key-and-adding-it-to-the-ssh-agent"
    default_severity = Severity.INFO
    default_description = "Detects when a GitHub user access key is created."
    tests = git_hub_user_access_key_created_tests

    def rule(self, event):
        return event.get("action") == "public_key.create"

    def title(self, event):
        return f"User [{event.udm('actor_user')}] created a new ssh key"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

gcp_storage_hmac_keys_create_tests: list[RuleTest] = [
    RuleTest(
        name="privilege-escalation",
        expected_result=True,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": True, "permission": "storage.hmacKeys.create"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    RuleTest(
        name="fail",
        expected_result=False,
        log={
            "protoPayload": {
                "authorizationInfo": [{"granted": False, "permission": "storage.hmacKeys.create"}],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
]


@panther_managed
class GCPStorageHmacKeysCreate(Rule):
    id = "GCP.Storage.Hmac.Keys.Create-prototype"
    display_name = "GCP storage hmac keys create"
    default_description = "There is a feature of Cloud Storage, “interoperability”, that provides a way for Cloud Storage to interact with storage offerings from other cloud providers, like AWS S3. As part of that, there are HMAC keys that can be created for both Service Accounts and regular users. We can escalate Cloud Storage permissions by creating an HMAC key for a higher-privileged Service Account."
    log_types = [LogType.GCP_AUDIT_LOG]
    default_severity = Severity.HIGH
    default_reference = (
        "https://rhinosecuritylabs.com/cloud-security/privilege-escalation-google-cloud-platform-part-2/"
    )
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    tests = gcp_storage_hmac_keys_create_tests

    def rule(self, event):
        auth_info = event.deep_walk("protoPayload", "authorizationInfo", default=[])
        auth_info = auth_info if isinstance(auth_info, list) else [auth_info]
        for auth in auth_info:
            if auth.get("granted", False) and auth.get("permission", "") == "storage.hmacKeys.create":
                return True
        return False



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

gcp_workforce_pool_createdor_updated_tests: list[RuleTest] = [
    RuleTest(
        name="DeleteWorkforcePool-False",
        expected_result=False,
        log={
            "insertId": "1plwiv7e2lay7",
            "logName": "organizations/123456789012/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "locations/global/workforcePools/test-pool/operations/bigar3hp32vamefaukfkaaq000000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@example.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workforcePools.delete",
                        "resource": "locations/global/workforcePools/test-pool",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.admin.v1.WorkforcePools.DeleteWorkforcePool",
                "request": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.DeleteWorkforcePoolRequest",
                    "name": "locations/global/workforcePools/test-pool",
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-11-17T18:58:52.165673889Z",
                    },
                },
                "resourceName": "locations/global/workforcePools/test-pool",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:58:52.901258022Z",
            "resource": {
                "labels": {
                    "method": "google.iam.admin.v1.WorkforcePools.DeleteWorkforcePool",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:58:52.158942930Z",
        },
    ),
    RuleTest(
        name="UpdateWorkforcePool-True",
        expected_result=True,
        log={
            "insertId": "1h09dxwe33hgu",
            "logName": "organizations/123456789012/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "locations/global/workforcePools/test-pool/operations/bigarg7n32vamefy6ximiaq000000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@example.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workforcePools.update",
                        "resource": "locations/global/workforcePools/test-pool",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.admin.v1.WorkforcePools.UpdateWorkforcePool",
                "request": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.UpdateWorkforcePoolRequest",
                    "updateMask": "description,sessionDuration,disabled,displayName",
                    "workforcePool": {
                        "description": "Test pool to facilitate detection writing",
                        "displayName": "Test Pool",
                        "name": "locations/global/workforcePools/test-pool",
                        "sessionDuration": "43200s",
                    },
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-11-17T18:53:15.208909504Z",
                    },
                },
                "resourceName": "locations/global/workforcePools/test-pool",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:53:16.523653141Z",
            "resource": {
                "labels": {
                    "method": "google.iam.admin.v1.WorkforcePools.UpdateWorkforcePool",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:53:15.200613481Z",
        },
    ),
    RuleTest(
        name="CreateWorkforcePool-True",
        expected_result=True,
        log={
            "insertId": "6432zre32u1v",
            "logName": "organizations/123456789012/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "first": True,
                "id": "locations/global/workforcePools/test-pool/operations/bifqrwxk32vamegiyoqaoeab00000000",
                "producer": "iam.googleapis.com",
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@example.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iam.workforcePools.create",
                        "resource": "organizations/123456789012",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.iam.admin.v1.WorkforcePools.CreateWorkforcePool",
                "request": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.CreateWorkforcePoolRequest",
                    "location": "locations/global",
                    "workforcePool": {
                        "description": "Test pool",
                        "displayName": "Test Pool",
                        "name": "locations/global/workforcePools/test-pool",
                        "parent": "organizations/325169835352",
                        "sessionDuration": "3600s",
                        "state": "ACTIVE",
                    },
                    "workforcePoolId": "test-pool",
                },
                "requestMetadata": {
                    "callerIp": "07da:0994:97fb:8db1:c68f:c109:fcdd:d594",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:109.0) Gecko/20100101 Firefox/119.0,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {
                        "auth": {},
                        "reason": "8uSywAYQGg5Db2xpc2V1bSBGbG93cw",
                        "time": "2023-11-17T18:47:53.284817626Z",
                    },
                },
                "resourceName": "organizations/325169835352",
                "serviceName": "iam.googleapis.com",
            },
            "receiveTimestamp": "2023-11-17T18:47:54.138395349Z",
            "resource": {
                "labels": {
                    "method": "google.iam.admin.v1.WorkforcePools.CreateWorkforcePool",
                    "service": "iam.googleapis.com",
                },
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2023-11-17T18:47:53.276929945Z",
        },
    ),
]


@panther_managed
class GCPWorkforcePoolCreatedorUpdated(Rule):
    id = "GCP.Workforce.Pool.Created.or.Updated-prototype"
    display_name = "GCP Workforce Pool Created or Updated"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["Account Manipulation", "Additional Cloud Roles", "GCP", "Privilege Escalation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1136.003", "TA0003:T1098.003", "TA0004:T1098.003"]}
    default_severity = Severity.HIGH
    default_runbook = "Ensure that the Workforce Pool creation or modification was expected. Adversaries may use this to persist or allow additional access or escalate their privilege.\n"
    default_reference = (
        "https://medium.com/google-cloud/detection-of-inbound-sso-persistence-techniques-in-gcp-c56f7b2a588b"
    )
    tests = gcp_workforce_pool_createdor_updated_tests
    METHODS = [
        "google.iam.admin.v1.WorkforcePools.CreateWorkforcePool",
        "google.iam.admin.v1.WorkforcePools.UpdateWorkforcePool",
    ]

    def rule(self, event):
        return event.deep_get("protoPayload", "methodName", default="") in self.METHODS

    def title(self, event):
        actor = event.deep_get("protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        workforce_pool = event.deep_get("protoPayload", "request", "workforcePool", "name", default="").split("/")[-1]
        resource = organization_id = event.deep_get("logName", default="<LOG_NAME_NOT_FOUND>").split("/")
        organization_id = resource[resource.index("organizations") + 1]
        return (
            f"GCP: [{actor}] created or updated workforce pool [{workforce_pool}] in organization [{organization_id}]"
        )

    def alert_context(self, event):
        return event.deep_get("protoPayload", "request", "workforcePool", default={})



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tines_helpers import tines_alert_context

tines_actions_disabled_changes_tests: list[RuleTest] = [
    RuleTest(
        name="Tines Actions Disabled Change",
        expected_result=True,
        log={
            "created_at": "2023-05-23 23:16:41",
            "id": 7111111,
            "operation_name": "ActionsDisabledChange",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
    RuleTest(
        name="Tines Login",
        expected_result=False,
        log={
            "created_at": "2023-05-17 14:45:19",
            "id": 7888888,
            "operation_name": "Login",
            "request_ip": "12.12.12.12",
            "request_user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
            "tenant_id": "8888",
            "user_email": "user@company.com",
            "user_id": "17171",
            "user_name": "user at company dot com",
        },
    ),
]


@panther_managed
class TinesActionsDisabledChanges(Rule):
    id = "Tines.Actions.DisabledChanges-prototype"
    display_name = "Tines Actions Disabled Change"
    log_types = [LogType.TINES_AUDIT]
    tags = ["Tines"]
    default_reference = "https://www.tines.com/university/tines-basics/architecture-of-an-action"
    default_severity = Severity.MEDIUM
    default_description = "Detections when Tines Actions are set to Disabled Change\n"
    summary_attributes = ["user_id", "operation_name", "tenant_id", "request_ip"]
    tests = tines_actions_disabled_changes_tests
    ACTIONS = ["ActionsDisabledChange"]

    def rule(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        return action in self.ACTIONS

    def title(self, event):
        action = deep_get(event, "operation_name", default="<NO_OPERATION_NAME>")
        actor = deep_get(event, "user_email", default="<NO_USERNAME>")
        return f"Tines: {action} by {actor}"

    def alert_context(self, event):
        return tines_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_auth_factor_removed_tests: list[RuleTest] = [
    RuleTest(
        name="User removed an auth factor",
        expected_result=True,
        log={
            "event_type_id": "172",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "authentication_factor_description": "2FA Name",
        },
    ),
    RuleTest(
        name="User deactivated an otp deice",
        expected_result=True,
        log={
            "event_type_id": "24",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
            "otp_device_name": "2FA Name",
        },
    ),
]


@panther_managed
class OneLoginAuthFactorRemoved(Rule):
    id = "OneLogin.AuthFactorRemoved-prototype"
    display_name = "OneLogin Authentication Factor Removed"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Identity & Access Management", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_severity = Severity.LOW
    default_description = "A user removed an authentication factor or otp device.\n"
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010426"
    default_runbook = "Investigate whether this was an intentional action and if other multifactor devices exist.\n"
    summary_attributes = [
        "account_id",
        "event_type_id",
        "user_name",
        "user_id",
        "authentication_factor_description",
        "otp_device_name",
    ]
    tests = one_login_auth_factor_removed_tests

    def rule(self, event):
        # verify this is a auth factor being removed
        # event id 24 is otp device deregistration
        # event id 172 is a user deleted an authentication factor
        return str(event.get("event_type_id")) == "24" or str(event.get("event_type_id")) == "172"

    def dedup(self, event):
        return event.get("user_name", "<UNKNOWN_USER>")

    def title(self, event):
        if str(event.get("event_type_id")) == "172":
            return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] removed an authentication factor [{event.get('authentication_factor_description', '<UNKNOWN_AUTH_FACTOR>')}]"
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] deactivated an otp device [{(event.get('otp_device_name', '<UNKNOWN_OTP_DEVICE>'),)}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsec2_stop_instances_tests: list[RuleTest] = [
    RuleTest(
        name="CloudTrail Instances Were Stopped",
        expected_result=True,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "EXAMPLE6E4XEGITWATV6R",
                "arn": "arn:aws:iam::777788889999:user/Nikki",
                "accountId": "777788889999",
                "accessKeyId": "AKIAI44QH8DHBEXAMPLE",
                "userName": "Nikki",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-07-19T21:11:57Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-07-19T21:14:20Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "StopInstances",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/2.13.5 Python/3.11.4 Linux/4.14.255-314-253.539.amzn2.x86_64 exec-env/CloudShell exe/x86_64.amzn.2 prompt/off command/ec2.stop-instances",
            "requestParameters": {
                "instancesSet": {
                    "items": [{"instanceId": "i-EXAMPLE56126103cb"}, {"instanceId": "i-EXAMPLEaff4840c22"}],
                },
                "force": False,
            },
            "responseElements": {
                "requestId": "c308a950-e43e-444e-afc1-EXAMPLE73e49",
                "instancesSet": {
                    "items": [
                        {
                            "instanceId": "i-EXAMPLE56126103cb",
                            "currentState": {"code": 64, "name": "stopping"},
                            "previousState": {"code": 16, "name": "running"},
                        },
                        {
                            "instanceId": "i-EXAMPLEaff4840c22",
                            "currentState": {"code": 64, "name": "stopping"},
                            "previousState": {"code": 16, "name": "running"},
                        },
                    ],
                },
            },
            "requestID": "c308a950-e43e-444e-afc1-EXAMPLE73e49",
            "eventID": "9357a8cc-a0eb-46a1-b67e-EXAMPLE19b14",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "777788889999",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.2",
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "ec2.us-east-1.amazonaws.com",
            },
            "sessionCredentialFromConsole": "true",
        },
    ),
    RuleTest(
        name="CloudTrail Instances Were Started",
        expected_result=False,
        log={
            "eventVersion": "1.08",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "EXAMPLE6E4XEGITWATV6R",
                "arn": "arn:aws:iam::123456789012:user/Mateo",
                "accountId": "123456789012",
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "userName": "Mateo",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-07-19T21:11:57Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-07-19T21:17:28Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "StartInstances",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/2.13.5 Python/3.11.4 Linux/4.14.255-314-253.539.amzn2.x86_64 exec-env/CloudShell exe/x86_64.amzn.2 prompt/off command/ec2.start-instances",
            "requestParameters": {
                "instancesSet": {
                    "items": [{"instanceId": "i-EXAMPLE56126103cb"}, {"instanceId": "i-EXAMPLEaff4840c22"}],
                },
            },
            "responseElements": {
                "requestId": "e4336db0-149f-4a6b-844d-EXAMPLEb9d16",
                "instancesSet": {
                    "items": [
                        {
                            "instanceId": "i-EXAMPLEaff4840c22",
                            "currentState": {"code": 0, "name": "pending"},
                            "previousState": {"code": 80, "name": "stopped"},
                        },
                        {
                            "instanceId": "i-EXAMPLE56126103cb",
                            "currentState": {"code": 0, "name": "pending"},
                            "previousState": {"code": 80, "name": "stopped"},
                        },
                    ],
                },
            },
            "requestID": "e4336db0-149f-4a6b-844d-EXAMPLEb9d16",
            "eventID": "e755e09c-42f9-4c5c-9064-EXAMPLE228c7",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "123456789012",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.2",
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "ec2.us-east-1.amazonaws.com",
            },
            "sessionCredentialFromConsole": "true",
        },
    ),
    RuleTest(
        name="Error Stopping CloudTrail Instances",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "SomeErrorCode",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "EXAMPLE6E4XEGITWATV6R",
                "arn": "arn:aws:iam::777788889999:user/Nikki",
                "accountId": "777788889999",
                "accessKeyId": "AKIAI44QH8DHBEXAMPLE",
                "userName": "Nikki",
                "sessionContext": {
                    "sessionIssuer": {},
                    "webIdFederationData": {},
                    "attributes": {"creationDate": "2023-07-19T21:11:57Z", "mfaAuthenticated": "false"},
                },
            },
            "eventTime": "2023-07-19T21:14:20Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "StopInstances",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "192.0.2.0",
            "userAgent": "aws-cli/2.13.5 Python/3.11.4 Linux/4.14.255-314-253.539.amzn2.x86_64 exec-env/CloudShell exe/x86_64.amzn.2 prompt/off command/ec2.stop-instances",
            "requestParameters": {
                "instancesSet": {
                    "items": [{"instanceId": "i-EXAMPLE56126103cb"}, {"instanceId": "i-EXAMPLEaff4840c22"}],
                },
                "force": False,
            },
            "requestID": "c308a950-e43e-444e-afc1-EXAMPLE73e49",
            "eventID": "9357a8cc-a0eb-46a1-b67e-EXAMPLE19b14",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "managementEvent": True,
            "recipientAccountId": "777788889999",
            "eventCategory": "Management",
            "tlsDetails": {
                "tlsVersion": "TLSv1.2",
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "ec2.us-east-1.amazonaws.com",
            },
            "sessionCredentialFromConsole": "true",
        },
    ),
]


@panther_managed
class AWSEC2StopInstances(Rule):
    id = "AWS.EC2.StopInstances-prototype"
    display_name = "CloudTrail EC2 StopInstances"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["panther-signal"]
    default_severity = Severity.INFO
    default_description = "A CloudTrail instances were stopped. It makes further changes of instances possible\n"
    default_reference = "https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-log-file-examples.html"
    tests = awsec2_stop_instances_tests

    def rule(self, event):
        return all(
            [not event.get("errorCode"), not event.get("errorMessage"), event.get("eventName") == "StopInstances"],
        )

    def title(self, event):
        instances = [
            instance["instanceId"]
            for instance in event.deep_get("requestParameters", "instancesSet", "items", default=[])
        ]
        account = event.get("recipientAccountId")
        return f"EC2 instances {instances} stopped in account {account}."

    def alert_context(self, event):
        context = aws_rule_context(event)
        context["instance_ids"] = [
            instance["instanceId"]
            for instance in event.deep_get("requestParameters", "instancesSet", "items", default=[])
        ]
        return context



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get


def get_event_type(event):
    # pylint: disable=too-many-return-statements
    etype = deep_get(event, "event", "type")
    return {
        "user.login": event_type.SUCCESSFUL_LOGIN,
        "user.logout": event_type.SUCCESSFUL_LOGOUT,
        "user.settings.email_updated": event_type.USER_ACCOUNT_MODIFIED,
        "user.settings.login_method.mfa_backup_code_updated": event_type.MFA_RESET,
        "user.settings.login_method.mfa_totp_updated": event_type.MFA_RESET,
        "user.settings.login_method.password_added": event_type.USER_ACCOUNT_MODIFIED,
        "user.settings.preferred_name_updated": event_type.USER_ACCOUNT_MODIFIED,
        "user.settings.profile_photo_updated": event_type.USER_ACCOUNT_MODIFIED,
        "workspace.permissions.member_role_updated": event_type.USER_ROLE_MODIFIED,
    }.get(etype, etype)


def get_actor_user(event):
    actor = deep_get(event, "event", "actor", "id", default="UNKNOWN USER")
    if deep_get(event, "event", "actor", "person"):
        actor = deep_get(event, "event", "actor", "person", "email", default="UNKNOWN USER")
    return actor


class StandardNotionAuditLogs(DataModel):
    id: str = "Standard.Notion.AuditLogs"
    display_name: str = "Notion Audit Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.NOTION_AUDIT_LOGS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", method=get_actor_user),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="$.event.ip_address"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_audit_admin_grant_tests: list[RuleTest] = [
    RuleTest(
        name="Super Admin granted",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Super Admin",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Admin granted",
        expected_result=True,
        log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Administrator",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
    RuleTest(
        name="Other role granted",
        expected_result=False,
        log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
]


@panther_managed
class CarbonBlackAuditAdminGrant(Rule):
    id = "CarbonBlack.Audit.Admin.Grant-prototype"
    log_types = [LogType.CARBON_BLACK_AUDIT]
    default_description = "Detects when a user is granted Admin or Super Admin permissions."
    display_name = "Carbon Black Admin Role Granted"
    default_severity = Severity.HIGH
    tags = ["Privilege Escalation", "Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0004:T1098"]}
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-CF5ACD2C-A534-46C8-AE06-E1884DB37B58.html"
    tests = carbon_black_audit_admin_grant_tests
    PREFIXES = ("Updated grant: ", "Created grant: ")

    def rule(self, event):
        desc = event.get("description", "")
        return all(
            [
                event.get("requestUrl", "").startswith("/access/"),
                any(desc.startswith(prefix) for prefix in self.PREFIXES),
                "Admin" in desc,
            ],
        )

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"

    def severity(self, event):
        if "Super Admin" in event.get("description", ""):
            return "CRITICAL"
        return "HIGH"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_zoom_helpers import get_zoom_usergroup_context as get_context

zoom_passcode_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Meeting Passcode Disabled",
        expected_result=True,
        log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Update",
            "operation_detail": "Edit Group Springfield  - Personal Meeting ID (PMI) Passcode: from On to Off",
            "p_log_type": "Zoom.Operation",
        },
    ),
    RuleTest(
        name="Meeting Passcode Enabled",
        expected_result=False,
        log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Update",
            "operation_detail": "Edit Group Springfield  - Personal Meeting ID (PMI) Passcode: from Off to On",
            "p_log_type": "Zoom.Operation",
        },
    ),
    RuleTest(
        name="Add User Group",
        expected_result=False,
        log={
            "time": "2021-11-17 00:37:24Z",
            "operator": "homer@panther.io",
            "category_type": "User Group",
            "action": "Add",
            "operation_detail": "Add Group Engineers",
            "p_log_type": "Zoom.Operation",
        },
    ),
]


@panther_managed
class ZoomPasscodeDisabled(Rule):
    id = "Zoom.PasscodeDisabled-prototype"
    display_name = "Zoom Meeting Passcode Disabled"
    log_types = [LogType.ZOOM_OPERATION]
    tags = ["Zoom", "Collection:Video Capture"]
    default_severity = Severity.LOW
    default_description = "Meeting passcode requirement has been disabled from usergroup\n"
    reports = {"MITRE ATT&CK": ["TA0009:T1125"]}
    default_reference = "https://support.zoom.us/hc/en-us/articles/360033559832-Zoom-Meeting-and-Webinar-passcodes"
    default_runbook = (
        "Follow up with user or Zoom admin to ensure this meeting room's use case does not allow a passcode.\n"
    )
    summary_attributes = ["p_any_emails"]
    tests = zoom_passcode_disabled_tests

    def rule(self, event):
        if event.get("category_type") != "User Group":
            return False
        context = get_context(event)
        changed = "Passcode" in context.get("Change", "")
        disabled = context.get("DisabledSetting", False)
        return changed and disabled

    def title(self, event):
        context = get_context(event)
        return f"Group {context['GroupName']} passcode requirement disabled by {event.get('operator')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success

awskms_customer_managed_key_loss_tests: list[RuleTest] = [
    RuleTest(
        name="KMS Key Disabled",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "DisableKey",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {"keyId": "1"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                },
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="KMS Key Scheduled For Deletion",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "ScheduleKeyDeletion",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {"keyId": "1", "pendingWindowInDays": 30, "overridePendingWindowCheck": False},
            "responseElements": {
                "keyId": "arn:aws:kms:us-west-2:123456789012:key/1",
                "deletionDate": "Jan 1, 2019 12:00:00 AM",
            },
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                },
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="KMS Key Non Deletion Event",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "Decrypt",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "encryptionContext": {
                    "aws:lambda:FunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:function",
                },
            },
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                },
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="KMS Key Scheduled For Deletion - missing resources",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "ScheduleKeyDeletion",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {"keyId": "1", "pendingWindowInDays": 30, "overridePendingWindowCheck": False},
            "responseElements": {
                "keyId": "arn:aws:kms:us-west-2:123456789012:key/1",
                "deletionDate": "Jan 1, 2019 12:00:00 AM",
            },
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "resources": None,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
            "p_row_id": "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234",
        },
    ),
    RuleTest(
        name="KMS Disable Key Error",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "NotFoundException",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "kms.amazonaws.com",
            "eventName": "DisableKey",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {"keyId": "1"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                },
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSKMSCustomerManagedKeyLoss(Rule):
    id = "AWS.KMS.CustomerManagedKeyLoss-prototype"
    display_name = "KMS CMK Disabled or Deleted"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity & Access Management", "Impact:Data Destruction"]
    reports = {"CIS": ["3.7"], "MITRE ATT&CK": ["TA0040:T1485"]}
    default_severity = Severity.INFO
    default_description = "A KMS Customer Managed Key was disabled or scheduled for deletion. This could potentially lead to permanent loss of encrypted data.\n"
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-kms-cmk-loss"
    default_reference = "https://docs.aws.amazon.com/kms/latest/developerguide/deleting-keys.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = awskms_customer_managed_key_loss_tests
    # API calls that are indicative of KMS CMK Deletion
    KMS_LOSS_EVENTS = {"DisableKey", "ScheduleKeyDeletion"}
    KMS_KEY_TYPE = "AWS::KMS::Key"

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.KMS_LOSS_EVENTS

    def dedup(self, event):
        for resource in event.get("resources") or []:
            if resource.get("type", "") == self.KMS_KEY_TYPE:
                return resource.get("ARN")
        return event.get("eventName")

    def alert_context(self, event):
        return aws_rule_context(event)
